﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Collections.Specialized;
using System.Configuration.Provider;
using System.Web;
using System.Configuration;
using System.Web.Configuration;
using Microsoft.ApplicationServer.Caching;

namespace ReusableCache.Provider
{
    #region provider model
    public class ReusableAppFabricCacheProvider : ReusableCacheProvider
    {

        private object _lock = new object();
        private string region = null;
        private readonly DataCacheFactory cacheFactory;
        private DataCache Cache
        {
            get
            {
                if (HttpContext.Current != null)
                {
                    return (new DataCacheFactory()).GetDefaultCache();
                }
                else
                {
                    return null;
                }

            }
        }

        public override void Put(string key, object value, TimeSpan slidingExpiration)
        {
            Cache.Put(key, value, slidingExpiration);
        }

        public override object Get(string key)
        {
            if (Cache != null)
            {
                return Cache.Get(key);
            }

            return null;
        }
    }

    public class ReusableWebCacheProvider : ReusableCacheProvider
    {

        private object _lock = new object();
        private System.Web.Caching.Cache Cache
        {
            get
            {
                if (HttpContext.Current != null)
                {
                    return HttpContext.Current.Cache;
                }
                else
                {
                    return null;
                }

            }
        }

        public override void Put(string key, object value, TimeSpan slidingExpiration)
        {
            Cache.Add(key, value, null, System.Web.Caching.Cache.NoAbsoluteExpiration, slidingExpiration, System.Web.Caching.CacheItemPriority.Normal, null);
        }

        public override object Get(string key)
        {
            if (Cache != null)
            {
                return Cache[key];
            }

            return null;
        }
    }

    public abstract class ReusableCacheProvider : ProviderBase
    {
        public override void Initialize(string name, NameValueCollection config)
        {
            if (config == null)
            {
                throw new ArgumentNullException("config");
            }

            if (config.Count > 0)
            {
                if (config["timeout"] != null)
                {
                    int cacheTime = 0;
                    if (int.TryParse(config["timeout"], out cacheTime))
                    {
                        Timeout = cacheTime;
                    }
                }
            }

            base.Initialize(name, config);
        }
        public int Timeout { get; set; }
        public abstract void Put(string key, object value, TimeSpan slidingExpiration);
        public abstract object Get(string key);
    }
    public class ReusableCacheConfiguration : ConfigurationSection
    {
        [ConfigurationProperty("providers")]
        public ProviderSettingsCollection Providers
        {
            get
            {
                return (ProviderSettingsCollection)base["providers"];
            }
        }

        [ConfigurationProperty("settings")]
        public NameValueConfigurationCollection Settings
        {
            get
            {
                return (NameValueConfigurationCollection)base["settings"];
            }
        }

        [ConfigurationProperty("defaultProvider", DefaultValue = "")]
        public string Default
        {
            get
            {
                return (string)base["defaultProvider"];
            }
            set
            {
                base["defaultProvider"] = value;
            }
        }
    }
    public class ReusableCacheProviderCollection : ProviderCollection
    {
        new public ReusableCacheProvider this[string name]
        {
            get { return (ReusableCacheProvider)base[name]; }
        }

        public override void Add(ProviderBase provider)
        {
            if (provider == null)
                throw new ArgumentNullException("provider");

            if (!(provider is ReusableCacheProvider))
                throw new ArgumentException("Invalid provider type", "provider");

            base.Add(provider);
        }
    }
    public class ReusableCacheProviderManager
    {
        private static ReusableCacheProviderCollection _providers;
        private static ReusableCacheProvider _defaultProvider;

        static ReusableCacheProviderManager()
        {
            Initialize();
        }

        private static void Initialize()
        {
            ReusableCacheConfiguration configuration = (ReusableCacheConfiguration)ConfigurationManager.GetSection("reusableCacheProvider");

            if (configuration == null)
                throw new ConfigurationErrorsException("cacheConfiguration configuration section is not set correctly.");

            _providers = new ReusableCacheProviderCollection();

            ProvidersHelper.InstantiateProviders(configuration.Providers, _providers, typeof(ReusableCacheProvider));

            _providers.SetReadOnly();

            _defaultProvider = _providers[configuration.Default];

            if (_defaultProvider == null)
                throw new ConfigurationErrorsException("default cache provider is not specified.", configuration.ElementInformation.Properties["defaultProvider"].Source, configuration.ElementInformation.Properties["defaultProvider"].LineNumber);


        }

        public static ReusableCacheProvider Provider
        {
            get
            {
                return _defaultProvider;
            }
        }

        public static ReusableCacheProviderCollection Providers
        {
            get
            {
                return _providers;
            }
        }
    }
    #endregion
}
